function [pw_spectrum, bpms, afcost, estimate] = Umojatest(dataset, params, setstr, xlims)

DopplerData = dataset;
times = DopplerData.times;

% Calculations per second
steps = params.steps;

fprintf('Signal samples: %d\n', length(DopplerData.data));
fprintf('Signal duration: %f s\n', times(end) - times(1));
%fprintf('Segment duration: %f s\n', xlims(2) - xlims(1));

%% Get Doppler data
doppler = double(DopplerData.data);

% Sampling frequency 
%fs = length(doppler) / (times(end) - times(1));
% Use known fs
if (strcmp(setstr, '006') || strcmp(setstr, '0010'))
	fs = 2572;
else
	fs = 2550;
end

fprintf('PRF: %f\n', fs);

%% Parameters

% Number of samples to average over
N = params.N;

% Power threshold
pthreshold = params.pthreshold;

% Decimation factor
dec = params.dec;

% Start BPM
%bpm_start = 140; 

% BPM range
bpm_max = params.bpm_max;                                  
bpm_min = params.bpm_min;

% Cycles per segment
cycles = params.cycles;  

%% Mean angular frequency estimate

% HP cutoff
%hpcutoff = 0.4;
%ripple = 2; % passband ripple in dB

%[b, a] = butter(16, hpcutoff, 'high');
%doppler_f = filter(b, a, doppler);

%[b, a] = cheby1(4, ripple, hpcutoff, 'high');
%doppler_f = filter(b, a, doppler);

if (params.highpass)
    doppler_f = cheby(doppler, params.chebystop, params.chebypass);
else
    if (params.envelope)
        doppler = abs(doppler + hilbert(real(doppler)));
    end
    [b, a] = butter(10, 0.04, 'low');
    doppler_f = filter(b, a, doppler);
end

for n = 1 : length(doppler_f) - 1 - N
    % Mean power estimate
    doppler_mp(n) = 20 * mean(log10(doppler_f(n : n + 1 + N)));
    % Mean frequency estimate
    R1(n) = mean(doppler_f(n + 1 : n + 1 + N) .* conj(doppler_f(n : n + N)));
end

if (params.envelope)
    for n = 1 : length(doppler_f) - 1 - N
        R1(n) = mean(doppler_f(n : n + 1 + N));
    end
end

if (params.antialias)
    % Lowpass filtering to prevent aliasing
    b = fir1(60, 1 / dec, 'low');
    R1_lp = filter2(b, R1);
    R1 = R1_lp;
end

% Thresholding based on power estimate
R1t = R1;
for n = 1 : length(doppler_f) - 1 - N
    if (abs(doppler_mp(n)) < pthreshold)
        R1t(n) = complex(0, 0);
    end
end

%{
% Plot power estimate with threshold
figure(200 + str2num(setstr))
t = linspace(times(1), times(end), length(doppler_mp));
pt = pthreshold * ones(1, length(doppler_mp));
subplot(2, 1, 1)
plot(t, abs(doppler_mp), '-r', t, max(abs(doppler_mp), pthreshold), '-b', t, pt, '-k')
xlim(xlims)
xlabel('time [s]')
ylabel('power [dB]')
title('Power estimate with threshold (75 dB)')
%ylim([-pi pi])
subplot(2, 1, 2)
plot(t, angle(R1t))
xlim(xlims)
xlabel('time [s]')
ylabel('angle [rad]')
title('Angular frequency estimate after thresholding')
ylim([-pi pi])
%}

%{
figure(200 + setnum)
t = linspace(times(1), times(end), length(R1));
subplot(2, 1, 1)
plot(t, angle(R1))
xlim(xlims)
xlabel('time [s]')
ylabel('angle [rad]')
title('Angular frequency estimate')
ylim([-pi pi])
subplot(2, 1, 2)
plot(t, angle(R1t))
xlim(xlims)
xlabel('time [s]')
ylabel('angle [rad]')
title('Angular frequency estimate after thresholding')
ylim([-pi pi])
%}

%% Doppler spectrum

% Light clutter filtering
if (params.highpass)
    [b,a] = butter(2, 0.4, 'high');
    iq_f = filter(b, a, double(DopplerData.data));
else
    [b, a] = butter(2, 0.04, 'low');
    iq_f = filter(b, a, double(DopplerData.data));    
end

pw_spectrum = [];

if (params.plotbpms)
    % Window length
    W = 64;
    % Overlap percent
    overlap = 0.75;
    % Window function
    ham = hamming(W)';

    step = round((1 - overlap) * W);

    col = 1;
    for i = 0 : step : length(DopplerData.data) - W
        pw_spectrum(:, col) = fftshift(fft(ham .* iq_f(i + 1 : i + W), 256))';
        %x = abs(pw_spectrum(:, col));
        %pw_avg(col) = 50 * mean(x' .* linspace(-1, 1, length(x))) / mean(x);
        col = col + 1;
    end
end

%% LP filter and decimation

% remove remaining noise in the estimate
%Ns = 60;
%R1lp = filter2(1 / Ns * ones(1, Ns), R1t);
%R1t = R1lp;

%figure(999 + setnum)
%freqz(ones(1, Ns), 1)

%figure(999)
%freqz(b, 1)

%{
if (strcmp(setstr, '0010'))
    figure(200 + setnum)
    t = linspace(times(1), times(end), length(R1));
    subplot(2, 1, 1)
    plot(t, angle(R1t))
    xlim(xlims)
    xlabel('time [s]')
    ylabel('angle [rad]')
    title('Angular frequency estimate after thresholding')
    ylim([-pi pi]) 
    subplot(2, 1, 2)
    plot(t, angle(R1lp))
    xlim(xlims)
    xlabel('time [s]')
    ylabel('angle [rad]')
    title('Angular frequency estimate after lowpass filtering')
    ylim([-pi pi]) 
end
%}

% New sampling rate after decimation
fsd = fs / dec;

R1_avg_dec = R1t(1 : dec : end);

estimate = [];

if (params.plotbpms)
    estimate = angle(R1_avg_dec);
    if (params.envelope)
        estimate = R1_avg_dec;
    end
end


%% Initialize variables

bpms = [];
% Start position
pos = 1;                                        
j = 1;

% Starting window length
siglength = round(cycles * 60 * fsd / bpm_min);
fprintf('Segment size: %d samples\n', siglength);

% Calculations per second
stepsize = round(fsd / steps);

prevmax = round(60 * fsd / bpm_min);
trisize = siglength;

%% Step through signal

count = 0;
while pos + siglength < length(R1_avg_dec) - N - 1

    %fprintf('Segment length, iteration %d: %d\n', count, siglength);
    %fprintf('Window length: %d\n', siglength);
    
    % Get signal segment
    %signal = R1_angle(pos : pos + siglength - 1); % when N > 1
    %signal = angle(R1_avg_dec(pos : pos + siglength - 1));
    count = count + 1;
    segment = R1_avg_dec(pos : pos + siglength - 1);
    signal_est = zeros(1, length(segment));
    
    % Normalize each segment
    segnorm = segment / max(max(segment), 1);
    segment = segnorm;
    
    signal_est = angle(segment);
    
    if (params.envelope)
        signal_est = segment;
    end
    signal_f = signal_est;
    
    if (params.saveaf)
        t1 = pos / fsd;
        t2 = (pos + siglength - 1) / fsd;
        segaxis = linspace(t1, t2, length(signal_f));
        config = [num2str(params.pthreshold), 'dB-', num2str(params.dec), 'dec-', num2str(params.cycles), 'c-tri', num2str(params.triwindow), '-aa', num2str(params.antialias), '-N', num2str(params.N)];
        save(['seg/segment-', setstr, '-', num2str(j), '-', config, '.mat'], 'signal_f', 'segaxis');
    end
    
        %figure(999 + j)
        %subplot(2,1,1)
        %plot(1 : length(segment), signal_r, '-b', 1 : length(segment), signal_i, '-r')
        %legend(['Real'; 'Imag'])
        %axis tight
        %subplot(2,1,2)
        %plot(signal_f)
        %ylim([-pi pi])
        %axis tight
        %title(['Angular frequency estimate, set ', setstr, ', segment ', num2str(j)])
        %fprintf('Segment length: %d\n', length(segment))
    
    
    % Get maximum of autocorrelation function
    [xcorr_max, cost] = getxcorrmax(signal_f, fsd, j, setstr, prevmax, params);
    
    %fprintf('AF maximum: %d\n', xcorr_max);
    
    %afreq = 2 * pi * fs / xcorr_max;
    %fprintf('Angular frequency: %f\n', afreq);
    
    % Segment length in samples based on previous bpm   
    %siglength = round(cycles * xcorr_max);
    %fprintf('Max element: %d\n', xcorr_max);
    %fprintf('Next segment size: %d\n', siglength);
    
    % Calculate BPM
    bpm = 60 * fsd / xcorr_max;
    % Heart period
    dt = xcorr_max / fsd;
    % Time index
    % Should be center of segment?
    t = (pos + siglength - 1) / fsd;
    %fprintf('BPM: %.1f\n', bpm);
    % Add BPM to list
    afcost(j) = cost;
    bpms(j, :) = [t + times(1) dt bpm];
    j = j + 1;
    
    % Increment position
    pos = pos + stepsize;

    % Save previous AF maximum
    prevmax = xcorr_max;
    % Use previous window size for triangle window length
    %trisize = siglength;
end

