function [dist, cost] = getxcorrmax(signal, fs, segnum, setstr, prevmax, params)

% trisize   Size of triangle window (if even, add one)
% prevmax   Previously calculated samples until max correlation

showplot = 0;
%showplot = (strcmp(setstr, '04') && (segnum == 4 || segnum == 8 || segnum == 11)) || (strcmp(setstr, '06') && (segnum == 10 || segnum == 12)) || (strcmp(setstr, '006') && (segnum == 26 || segnum == 35 || segnum == 22));
%showplot = (strcmp(setstr, '0010') && segnum == 11);
%showplot = strcmp(setstr, '04') && (segnum == 21 || segnum == 23 || segnum == 24);

windowing = params.triwindow;
bpm_min = params.bpm_min;
bpm_max = params.bpm_max;

% Make sure triangle window size is odd (for alignment)
%trisize = trisize + (mod(trisize, 2) == 0);
trisize = length(signal) + (mod(length(signal), 2) == 0);

% Show BPM on x-axis
x = 1 : length(signal);
xaxis = 60 * fs ./ x;

xcorr_signal = xcorr(signal, 'biased');

% We only need second half of the signal (symmetry)
xcorr_half_size = round((length(xcorr_signal) + 1) / 2);
xcorr_half = xcorr_signal(xcorr_half_size : length(xcorr_signal));

%fprintf('Max element: %d\n', max_elem + 100);
%fprintf('Max value: %f\n', max_test);

% Calculate correlation interval based on BPM range
mincorr = round(60 * fs / bpm_max);
maxcorr = round(60 * fs / bpm_min);

if (showplot)
    faf = figure(200 + segnum);
    subplot(2,1,1)
    plot(xcorr_half, 'r-');
    title('Autocorrelation function (full)')
    axis tight;
    grid on;
    xlabel('lag')
    ylabel('correlation') 
    set(faf,'Name',['Set ', setstr, segnum, ', segment ', segnum ,', angle, 80 dB, N=32'],'NumberTitle','off')
    sp = subplot(2,1,2);
end

if (windowing)
    %% Triangle window
    %C++ method
    tri_half = (trisize - 1) / 2;
    start_pos = prevmax - tri_half;
    last_pos = prevmax + tri_half + 1;

    trisig = zeros(1, length(xcorr_half));

    if (start_pos < 1)
        start_pos = 1;
    end
    if (last_pos >= length(signal))
        last_pos = length(signal);
    end

    peakval = 1;
    for i = start_pos : last_pos
        val = peakval - abs((i - 1) - 1 - prevmax) / tri_half;
        trisig(i) = val;
    end

    xcorr_win =  trisig .* xcorr_half;
else
    xcorr_win = xcorr_half; 
end

% limit the interval
maxcorr = min(maxcorr, length(xcorr_win));
xcorr_win = xcorr_win(mincorr : maxcorr);
xaxis = xaxis(mincorr : maxcorr);

% Plot triangular function
%{
if (showplot ~= 0)
    subplot(4,1,2)
    plot(x, trisig(1 : length(x)), x, xcorr_half / max(xcorr_half));
    title('Triangular window based on previous maximum')
    axis tight;
    grid on;
    xlabel('lag')

    % Plot resulting function
    subplot(4,1,3)
    plot(xaxis, xcorr_win);
    title('FHR likelihood function')
    axis tight;
    grid on;
    xlabel('BPM')
end
%}

% Find AF maximum
[max_val, dist] = max(xcorr_win);
energy = xcorr_win(1);

% Normalize the AF (power decreases with lag)
%{
xcorr_win_norm = zeros(1, length(xcorr_win));
samples = length(signal);
for i = 1 : length(xcorr_win)
    lag = i + mincorr;
    xcorr_win_norm(i) = xcorr_win(i) / (samples - lag);
end
[max_val_norm, dist_norm] = max(xcorr_win_norm);
%}

% Make sure detected maximum is a global maximum
[peaks, locs] = findpeaks(xcorr_win);
[maxpeak, maxloc] = max(peaks);

% Cost function
cost = 1 - abs(max_val) / xcorr_half(1);
%cost = 1 - abs(max_val - min(xcorr_win)) / xcorr_half(1);

%fprintf('Maximum peak: %f, location: %d\n', maxpeak, locs(maxloc));
%fprintf('Maximum value: %f, distance: %d\n', max_val_norm, dist_norm);

if (~isempty(maxpeak))
    max_val = maxpeak;
    dist = locs(maxloc);
else
    %fprintf('No peaks in segment %d\n', segnum);
    % There are no peaks, which means we can disregard the result
    cost = 1;
end

% Draw ideal peak locations in the plot (140 BPM)
pinterval = round(60 * fs / 140);
pthreshold = 0.3 * max_val;
segs = floor(length(xcorr_win) / pinterval);

refpeaks = pthreshold * ones(1, segs);
reflocs = pinterval * (1 : segs);

if (showplot) 
    delete(sp);    
    sp = subplot(2,1,2);
    hold on
    plot(linspace(mincorr, maxcorr, length(xcorr_win)), xcorr_win, 'm-');
    scatter(reflocs, refpeaks, 'filled', 'r');
    title('Autocorrelation function (limited)')
    axis tight;
    grid on;
    xlabel('lag')
    ylabel('correlation')
    xlim([mincorr maxcorr])
    hold off
    set(faf,'Name', ['Autocorrelation, imag(R1), set ', setstr, ', segment ', num2str(segnum)],'NumberTitle','off')
    %saveas(faf, ['fig/AF-set', setstr, '-seg', num2str(segnum), '-R1imag'], 'fig');
    
    %{
    subplot(3,1,3)
    plot(linspace(mincorr, maxcorr, length(xcorr_win_norm)), xcorr_win_norm, 'r-');
    axis tight;
    grid on;
    xlabel('lag')
    ylabel('correlation')
    xlim([mincorr maxcorr])    
    %}
    
    %pause(1)
end

if (params.saveaf)
    afaxis = linspace(mincorr, maxcorr, length(xcorr_win));
    bpmaxis = 60 * fs ./ afaxis;
    config = [num2str(params.pthreshold), 'dB-', num2str(params.dec), 'dec-', num2str(params.cycles), 'c-tri', num2str(params.triwindow), '-aa', num2str(params.antialias), '-N', num2str(params.N)];
    save(['seg/autocorr-', setstr, '-', num2str(segnum), '-', config,'.mat'], 'xcorr_win', 'afaxis', 'bpmaxis');
end


dist = dist + mincorr;

%fprintf('mincorr: %d, maxcorr: %d\n', mincorr, maxcorr);

%if (cost < 0.7)
%    fprintf('dist: %f, BPM: %f\n', dist, 60 * fs / dist);
%end

%fprintf('Distance (max): %d, Average: %d\n', dist, avgdist);
%dist = round(dist + avgdist / 2);

% Maximum-to-energy ratio

%fprintf('Signal energy: %f\n', xcorr_half(1));
%fprintf('Max value: %f\n', max_val);
